function [ksd,p,bootstrapSamples, info] = KSD_U_testBoot(x, dlogp, varargin)
% Estimate kernelized Stein discrepancy (KSD) using U-statistics, and use
% bootstrap for goodness of fit test H0: {x} is drawn from p(x) (KSD = 0)
%
% Inputs
%     --- X: sample of size Num_Instance X Num_Dimension
%     --- dlogp (function handle): the derivative of the log-density (score function) of p(x). It takes X as input and
%                   output a column vector of size Num_Instance * Dimension
%     --- additional optional parameters:
%            -- 'kernel': type of kernel (default: 'rbf'), 
%            -- 'bandwidth': bandwidth of the kernel; when 'width'=-1 or
%            'median' (default), we set it to be the median distance between the data
%            points.
%            --- 'nboot': bootstrap sample size. 
% 
% Outputs:
%     --- ksd: the estimated kernelized Stein discrpancy (KSD)
%     --- p  :  pvalue for rejecting H0: ksd=0
%     --- bootstrapSamples: the bootstrap sample
%     --- info: other information 
%
% 
% Qiang Liu @ Jan 2016


%%%%%%%%% processing inputs %%%%%%%%
[kernel, h, nboot] = process_varargin(varargin, 'kernel', 'rbf', {'bandwidth','width'}, -1, 'nboot', 0);
% If dlogp = 'gaussian', calculate the score function for standard Gaussian
% distribution
if isa(dlogp, 'char'), 
    switch lower(dlogp)
        case 'gaussian'
        dlogp = @(x)(-x); % standard Gaussian 
    end
end


%%%%%%%% set the bandwidth of RBF kernel %%%%%%%%
if (isa(h,'char')&&strcmp(h,'median')) || (h==-1), 
    h = sqrt(0.5*median_distance(x));  %rbf_dot has factor two in kernel
elseif h ==-2
    h = ratio_median_heuristic(x, dlogp);    
end

Sqx = dlogp(x);
n=size(x,1); dim = size(x,2);
switch lower(kernel)
    case 'rbf'    
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%% main part: calculating u_q(x,x') %%%%%%%%%
        XY = x*x';
        x2= sum(x.^2, 2);
        X2e = repmat(x2, 1, n);

        H = (X2e + X2e' - 2*XY);
        Kxy = exp(-H/(2*h^2));
        sqxdy = -(Sqx*x' - repmat(sum((Sqx.*x),2),1,n))./h^2;
        dxsqy = sqxdy';
        dxdy = (-H/h^4 + dim/h^2);
        M = (Sqx*Sqx' + sqxdy + dxsqy + dxdy).*Kxy;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                
    otherwise
        error('Sorry, RBF is the only option for now.');
end

M2 = M -diag(diag(M));
ksd = sum(sum(M2))/(n*(n-1)); % U-statistics
ksdV = sum(sum(M))/(n^2);   % V-statistics 

%% bootstrap for calculating the p-value: 
bootstrapSamples = zeros(nboot,1);

for i = 1:nboot        
    wtsboot= mnrnd(n,ones(1,n)/n)/n;     
    bootstrapSamples(i) = (wtsboot-1/n)*M2*(wtsboot' - 1/n);    
end
p = mean(  bootstrapSamples >= ksd );
        

info.bandwidth = h;
info.M = M;
info.nboot = nboot;
info.ksd_V = ksdV;

return;
end

