function [ksd, p, uq, info] = KSD_linear(xx, dlogp, varargin)
% Estimate kernelized Stein discrpancy using linear estimator, with
% Gaussian test for H0: {xx} is drawn from p(x) (ksd = 0)
%
%
% Inputs
%     --- xx: sample of size Num_Instance * Num_Dimension
%     --- dlogp (function handle): the derivative of the log-density (score function) of p(x). It takes xx as input and
%                   output a column vector of size Num_Instance * Dimension
%     --- additional optional parameters:
%            -- 'kernel': type of kernel (default: 'rbf'), 
%            -- 'bandwidth': bandwidth of the kernel; when 'width'=-1 or
%            'median' (default), we set it to be the median distance between the data
%            points.
% 
% Outputs:
%     --- ksd: the estimated kernelized Stein discrpancy (KSD)
%     --- p  :  pvalue for rejecting H0: ksd=0
%     --- uq: the sample of {kp(xi, xj)}
%     --- info: other information 

[kernel, h] = process_varargin(varargin, 'kernel', 'rbf', {'bandwidth','width'}, -1);      
if h==-1, 
    h = sqrt(0.5*median_distance(xx));  %rbf_dot has factor two in kernel
end

nhalf = floor(size(xx,1)/2); dim = size(xx,2);
x = xx(1:nhalf,:); y = xx(end-nhalf+1:end,:);
Sqx = dlogp(x); Sqy = dlogp(y);
 switch lower(kernel)
    case 'rbf'    
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        H = sum((x-y).^2,2);
        Kxy = exp(-H/(2*h^2));
        sqxsqy = sum(Sqx.*Sqy,2);
        sqxdy = sum(Sqx.*(x-y), 2)/h^2;
        dxsqy = sum((y-x).*Sqy, 2)/h^2;
        dxdy = -H/h^4  + dim/h^2;        

        uq = (sqxsqy + sqxdy + dxsqy + dxdy).*Kxy;        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    otherwise
        error('does not support this kernel right now');
end

ksd =mean(uq);
p = 1 - normcdf(ksd, 0, std(uq)/sqrt(nhalf));

info.bandwidth = h;

return;
end


