%This function implements the MMD two-sample test using a bootstrap
%approach to compute the test threshold.


%Arthur Gretton
%07/12/08

%Inputs: 
%        X contains dx columns, m rows. Each row is an i.i.d sample
%        Y contains dy columns, m rows. Each row is an i.i.d sample
%        alpha is the level of the test
%        params.sig is kernel size. If -1, use median distance heuristic.
%        params.shuff is number of bootstrap shuffles used to
%                     estimate null CDF
%        params.bootForce: if this is 1, do bootstrap, otherwise
%                     look for previously saved threshold


%Outputs: 
%        thresh: test threshold for level alpha test
%        testStat: test statistic: m * MMD_b (biased)



function [testStat,thresh,params, pvalue] = mmdTestBoot(X,Y,alpha,params);

    
m1=size(X,1);
m2 = size(Y,1);


%Set kernel size to median distance between points in aggregate sample
if params.sig == -1
  Z = [X;Y];  %aggregate the sample
  size1=size(Z,1);
    if size1>100
      Zmed = Z(1:100,:);
      size1 = 100;
    else
      Zmed = Z;
    end
    G = sum((Zmed.*Zmed),2);
    Q = repmat(G,1,size1);
    R = repmat(G',size1,1);
    dists = Q + R - 2*Zmed*Zmed';
    dists = dists-tril(dists);
    dists=reshape(dists,size1^2,1);
    params.sig = sqrt(0.5*median(dists(dists>0)));  %rbf_dot has factor two in kernel
end


K = rbf_dot(X,X,params.sig);
L = rbf_dot(Y,Y,params.sig);
KL = rbf_dot(X,Y,params.sig);


%MMD statistic. Here we use biased 
%v-statistic. NOTE: this is m * MMD_b (but is equivalent with the test based on U-statistics)
%testStat = 1/m * sum(sum(K + L - KL - KL'));
testStat = mean(mean(K)) + mean(mean(L)) - 2*mean(mean(KL));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%threshFileName = strcat('mmdTestThresh',num2str(m1), '_',num2str(m2));

Kz = [K KL; KL' L];

%if ~exist(strcat(threshFileName,'.mat'),'file') || params.bootForce==1  
%  disp(strcat('Generating new threshold: ',threshFileName))
  
  MMDarr = zeros(params.shuff,1);
  for whichSh=1:params.shuff
    
    [notUsed,indShuff] = sort(rand(m1+m2,1));
    KzShuff = Kz(indShuff,indShuff);
    K = KzShuff(1:m1,1:m1);
    L = KzShuff(m1+1:m1+m2,m1+1:m1+m2);
    KL = KzShuff(1:m1,m1+1:m1+m2);    
%    MMDarr(whichSh) = 1/m * sum(sum(K + L - KL - KL'));
    MMDarr(whichSh) = mean(mean(K)) + mean(mean(L)) - 2*mean(mean(KL));    
  end 

  MMDarr = sort(MMDarr);  
  thresh = MMDarr(round((1-alpha)*params.shuff));
  pvalue = mean(MMDarr >= testStat);
    
%  save(threshFileName,'thresh','MMDarr', 'pvalue');  

%else
%  load(threshFileName);
%end
